# Generated by roxygen2: do not edit by hand

export(Ml_CompRisk)
export(Ml_Cox)
export(Parameters_estimation)
export(Reml_CompRisk_frailty)
export(Reml_Cox_frailty)
export(check_data_format)
export(simulate_data)
importFrom(Matrix,Diagonal)
importFrom(Matrix,Matrix)
importFrom(Matrix,sparseMatrix)
importFrom(stats,pchisq)
importFrom(stats,rexp)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,uniroot)
